@@ -26,17 +26,17 @@
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "01/09/2017"
+__date__ = "26/01/2018"
 
 
 import unittest
 import logging
 import numpy
 
-from silx.test.utils import ParametricTestCase
+from silx.utils.testutils import ParametricTestCase
 from silx.gui.test.utils import SignalListener
 from silx.gui.test.utils import TestCaseQt
-from silx.test import utils
+from silx.utils import testutils
 from silx.utils import deprecation
 
 from silx.gui import qt
@@ -77,9 +77,9 @@ class TestPlotWidget(PlotWidgetTestCase, ParametricTestCase):
         self.assertEqual(self.plot.getYAxis().getLabel(), ylabel)
 
     def _checkLimits(self,
-                    expectedXLim=None,
-                    expectedYLim=None,
-                    expectedRatio=None):
+                     expectedXLim=None,
+                     expectedYLim=None,
+                     expectedRatio=None):
         """Assert that limits are as expected"""
         xlim = self.plot.getXAxis().getLimits()
         ylim = self.plot.getYAxis().getLimits()
@@ -132,13 +132,11 @@ class TestPlotWidget(PlotWidgetTestCase, ParametricTestCase):
 
         # Resize with aspect ratio
         self.plot.setKeepDataAspectRatio(True)
-        listener.clear()  # Clean-up received signal
         self.qapp.processEvents()
-        self.assertEqual(listener.callCount(), 0)  # No event when redrawing
+        listener.clear()  # Clean-up received signal
 
         self.plot.resize(200, 200)
         self.qapp.processEvents()
-
         self.assertNotEqual(listener.callCount(), 0)
 
 
@@ -723,7 +721,7 @@ class TestPlotAxes(TestCaseQt, ParametricTestCase):
                 if getter is not None:
                     self.assertEqual(getter(), expected)
 
-    @utils.test_logging(deprecation.depreclog.name, warning=2)
+    @testutils.test_logging(deprecation.depreclog.name, warning=2)
     def testOldPlotAxis_Logarithmic(self):
         """Test silx API prior to silx 0.6"""
         x = self.plot.getXAxis()
@@ -762,7 +760,7 @@ class TestPlotAxes(TestCaseQt, ParametricTestCase):
         self.assertEqual(self.plot.isYAxisLogarithmic(), False)
         self.assertEqual(listener.arguments(callIndex=-1), ("y", False))
 
-    @utils.test_logging(deprecation.depreclog.name, warning=2)
+    @testutils.test_logging(deprecation.depreclog.name, warning=2)
     def testOldPlotAxis_AutoScale(self):
         """Test silx API prior to silx 0.6"""
         x = self.plot.getXAxis()
@@ -801,7 +799,7 @@ class TestPlotAxes(TestCaseQt, ParametricTestCase):
         self.assertEqual(self.plot.isYAxisAutoScale(), True)
         self.assertEqual(listener.arguments(callIndex=-1), ("y", True))
 
-    @utils.test_logging(deprecation.depreclog.name, warning=1)
+    @testutils.test_logging(deprecation.depreclog.name, warning=1)
     def testOldPlotAxis_Inverted(self):
         """Test silx API prior to silx 0.6"""
         x = self.plot.getXAxis()
